import { IBBox } from "core/util/bbox";
import { SpatialIndex } from "core/util/spatial";
import { Glyph, GlyphView, GlyphData } from "./glyph";
import { Arrayable } from "core/types";
import { PointGeometry } from "core/geometry";
import { Context2d } from "core/util/canvas";
import { NumberSpec } from "core/vectorization";
import { LineMixinVector, FillMixinVector } from "core/property_mixins";
import { Line, Fill } from "core/visuals";
import { Selection } from "../selections/selection";
export interface MultiPolygonsData extends GlyphData {
    _xs: Arrayable<Arrayable<Arrayable<Arrayable<number>>>>;
    _ys: Arrayable<Arrayable<Arrayable<Arrayable<number>>>>;
    sxs: Arrayable<Arrayable<Arrayable<Arrayable<number>>>>;
    sys: Arrayable<Arrayable<Arrayable<Arrayable<number>>>>;
    hole_index: SpatialIndex;
}
export interface MultiPolygonsView extends MultiPolygonsData {
}
export declare class MultiPolygonsView extends GlyphView {
    model: MultiPolygons;
    visuals: MultiPolygons.Visuals;
    protected _index_data(): SpatialIndex;
    protected _index_hole_data(): SpatialIndex;
    protected _mask_data(): number[];
    protected _render(ctx: Context2d, indices: number[], { sxs, sys }: MultiPolygonsData): void;
    protected _hit_point(geometry: PointGeometry): Selection;
    private _get_snap_coord;
    scenterx(i: number, sx: number, sy: number): number;
    scentery(i: number, sx: number, sy: number): number;
    map_data(): void;
    draw_legend_for_index(ctx: Context2d, bbox: IBBox, index: number): void;
}
export declare namespace MultiPolygons {
    interface Mixins extends LineMixinVector, FillMixinVector {
    }
    interface Attrs extends Glyph.Attrs, Mixins {
        xs: NumberSpec;
        ys: NumberSpec;
    }
    interface Props extends Glyph.Props {
    }
    interface Visuals extends Glyph.Visuals {
        line: Line;
        fill: Fill;
    }
}
export interface MultiPolygons extends MultiPolygons.Attrs {
}
export declare class MultiPolygons extends Glyph {
    properties: MultiPolygons.Props;
    constructor(attrs?: Partial<MultiPolygons.Attrs>);
    static initClass(): void;
}
